/**********************************************************************
 * FILE: ama_scan.j
 * AUTHOR: Fabian Buske / Robert McLeay for refactoring
 * PROJECT: MEME
 * COPYRIGHT: 2007-2008, UQ
 * VERSION: $Revision: 1.0$
 * DESCRIPTION: Routines to perform average motif affinity scans
 *
 **********************************************************************/

#ifndef AMA_SCAN_H_
#define AMA_SCAN_H_

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include "matrix.h"
#include "meme-io.h"
#include "string-list.h"
#include "macros.h"
#include "motif.h"
#include "matrix.h"
#include "motif.h"
#include "array.h"
#include "seq.h"
#include "alphabet.h"
#include "cisml.h"
#include "zscore.h"
#include "assert.h"

extern char* motif_name; // Use this motif name in the output.

extern VERBOSE_T verbosity;

#define AVG_ODDS 0
#define MAX_ODDS 1

#define min(a,b)      (a<b)?a:b
#define max(a,b)      (a>b)?a:b

/*************************************************************************
 * Converts the motif frequency matrix into a odds matrix
 *************************************************************************/
void convert_to_odds_matrix(
	MOTIF_T* motif,		// motif to be converted
	ARRAY_T* bg_freqs	// background frequencies to use
);

/**********************************************************************
  ama_sequence_scan()

  scan a given sequence with a specified motif using either
  average motif affinity scoring or maximum one. In addition z-scores
  may be calculated.

  The motif has to be converted to log odds in advance (in order
  to speed up the scanning). Use convert_to_odds_matrix() once for each
  motif.

  The result will be stored in the scanned_sequence parameter
**********************************************************************/
void ama_sequence_scan(
	SEQ_T* sequence,	// the sequence to scan INPUT
	MOTIF_T* motif,		// the motif to scan with (converted to odds matrix) INPUT
	MOTIF_T* rev_motif, // the reversed motif
	int scoring,		// the scoring function to apply AVG_ODDS or MAX_ODDS INPUT
	int zscoring,		// the number of shuffled sequences used for z-score computation INPUT
	SCANNED_SEQUENCE_T* scanned_seq,	// where the results are put
	BOOLEAN_T scan_both_strands //Should we scan with both motifs and combine scores
);

/*************************************************************************
 * Calculate the odds score for each motif-sized window at each
 * site in the sequence using the given nucleotide frequencies.
 *
 * This function is a lightweight version based on the one contained in
 * motiph-scoring. Several calculations that are unnecessary for gomo
 * have been removed in order to speed up the process
 *************************************************************************/
double score_sequence(
		SEQ_T*        seq,				// sequence to scan (IN)
		MOTIF_T*      motif,				// motif already converted to log odds values (IN)
		char*         feature_type,		// motif name (IN)
		char*         seq_name,			// sequence name (IN)
		int 			method,				// method used for scoring (IN)
		SCANNED_SEQUENCE_T* scanned_seq	// the scanned sequence results (OUT)
		);

#endif /* AMA_SCAN_H_ */
